{% extends 'admin/base_site.html' %}

{% block content %}

<form action="" method="post">
  {% csrf_token %}
  <table>
    {{ form.as_table }}
  </table>
  <input type="submit" name="apply" value="Submit to Server"/>
</form>

{% endblock %}
